#categories and their order are defined here:
categories = ['Orchestra Syntax:Header',
              'Orchestra Syntax:Block Statements',
              'Orchestra Syntax:Macros',
              'Signal Generators:Additive Synthesis/Resynthesis',
              'Signal Generators:Basic Oscillators',
              'Signal Generators:Dynamic Spectrum Oscillators',
              'Signal Generators:FM Synthesis',
              'Signal Generators:Granular Synthesis',
              'Signal Generators:Hyper Vectorial Synthesis',
              'Signal Generators:Linear and Exponential Generators',
              'Signal Generators:Envelope Generators',
              'Signal Generators:Models and Emulations',
              'Signal Generators:Phasors',
              'Signal Generators:Random (Noise) Generators',
              'Signal Generators:Sample Playback',
              'Signal Generators:Scanned Synthesis',
              'Signal Generators:STK Opcodes',
              'Signal Generators:Table Access',
              'Signal Generators:Wave Terrain Synthesis',
              'Signal Generators:Waveguide Physical Modeling',
              'Signal I/O:File I/O',
              'Signal I/O:Signal Input',
              'Signal I/O:Signal Output',
              'Signal I/O:Software Bus',
              'Signal I/O:Printing and Display',
              'Signal I/O:Soundfile Queries',
              'Signal Modifiers:Amplitude Modifiers',
              'Signal Modifiers:Convolution and Morphing',
              'Signal Modifiers:Delay',
              'Signal Modifiers:Panning and Spatialization',
              'Signal Modifiers:Reverberation',
              'Signal Modifiers:Sample Level Operators',
              'Signal Modifiers:Signal Limiters',
              'Signal Modifiers:Special Effects',
              'Signal Modifiers:Standard Filters',
              'Signal Modifiers:Standard Filters:Resonant',
              'Signal Modifiers:Standard Filters:Control',
              'Signal Modifiers:Specialized Filters',
              'Signal Modifiers:Waveguides',
              'Signal Modifiers:Waveshaping',
              'Signal Modifiers:Comparators and Accumulators',
              'Instrument Control:Clock Control',
              'Instrument Control:Conditional Values',
              'Instrument Control:Compilation',
              'Instrument Control:Duration Control',
              'Instrument Control:Invocation',
              'Instrument Control:Program Flow Control',
              'Instrument Control:Realtime Performance Control',
              'Instrument Control:Initialization and Reinitialization',
              'Instrument Control:Sensing and Control',
              'Instrument Control:Stacks',
              'Instrument Control:Subinstrument Control',
              'Instrument Control:Time Reading',
              'Jacko Opcodes',
              'Lua Opcodes',
              'Serial I/O',
              'Table Control',
              'Table Control:Table Queries',
              'Table Control:Dynamic Selection',
              'Table Control:Read/Write Operations',
              'FLTK:Containers',
              'FLTK:Valuators',
              'FLTK:Other',
              'FLTK:Appearance',
              'Mathematical Operations:Arithmetic and Logic Operations',
              'Mathematical Operations:Arrays',
              'Mathematical Operations:Comparators and Accumulators',
              'Mathematical Operations:Mathematical Functions',
              'Mathematical Operations:Trigonometric Functions',
              'Mathematical Operations:Amplitude Functions',
              'Mathematical Operations:Random Functions',
              'Mathematical Operations:Opcode Equivalents of Functions',
              'Mathematical Operations:Vectors',
              'Pitch Converters:Functions',
              'Pitch Converters:Tuning Opcodes',
              'Real-time MIDI:Input',
              'Real-time MIDI:Output',
              'Real-time MIDI:Converters',
              'Real-time MIDI:Generic I/O',
              'Real-time MIDI:Event Extenders',
              'Real-time MIDI:Note Output',
              'Real-time MIDI:MIDI/Score Interoperability',
              'Real-time MIDI:System Realtime',
              'Real-time MIDI:Slider Banks',
              'Signal Flow Graph Opcodes',
              'Spectral Processing:STFT',
              'Spectral Processing:LPC',
              'Spectral Processing:Non-Standard',
              'Spectral Processing:Streaming',
              'Spectral Processing:ATS',
              'Spectral Processing:Loris',
              'Spectral Processing:Other',
              'Strings:Definition',
              'Strings:Manipulation',
              'Strings:Conversion',
              'Vectorial:Tables',
              'Vectorial:Scalar operations',
              'Vectorial:Vectorial operations',
              'Vectorial:Envelopes',
              'Vectorial:Limiting and Wrapping',
              'Vectorial:Delay Paths',
              'Vectorial:Random',
              'Vectorial:Cellular Automata',
              'Zak Patch System',
              'Plugin Hosting:DSSI and LADSPA',
              'OSC',
              'Faust Opcodes',
              'Network',
              'Remote Opcodes',
              'Mixer Opcodes',
              'Ableton Link Opcodes',
              'Python Opcodes',
              'Image Processing Opcodes',
              'Array Opcodes',
              'Array Operations: Fast Fourier Transform',
              'Array Operations: complex numbers',
              'Array Operations: Discrete Cosine Transform',
              'Array Operations: dot product',
              'Array Operations: Mel scale filterbank',
              'Array Operations: sorting',
              'Array Operations: Cepstrum',
              'Deprecated',
              'Miscellaneous',
              'Testing',
              'Utilities']
